package org.apidesign.apifest08.currency;

import java.util.Currency;
import java.util.HashMap;
import java.util.Map;

/**
 * A {@link Convertor} that works from a pre-set conversion table.
 * First use {@link #putIntoTable(org.apidesign.apifest08.currency.ConversionRate)} to set the conversion table.
 * Then invoke the {@link #convert(org.apidesign.apifest08.currency.Convertor.ConversionRequest)} method as many times as you wish.
 * @author jdvorak
 */
public class TableConvertor implements Convertor {

    private final Map<Currency, Map<Currency, ConversionRate>> conversionTable = new HashMap<Currency, Map<Currency, ConversionRate>>();
    
    public TableConvertor() {
    }

    /**
     * Puts a rate into the table.
     * @param rate
     */
    public void putIntoTable( final ConversionRate rate ) {
        final Currency srcCurrency = rate.getSrcUnitAmount().getCurrency();
        final Currency tgtCurrency = rate.getTgtUnitAmount().getCurrency();
        synchronized ( conversionTable ) {
            Map<Currency, ConversionRate> targetTable = conversionTable.get( srcCurrency );
            if ( targetTable == null ) {
                targetTable = new HashMap<Currency, ConversionRate>();
                conversionTable.put( srcCurrency, targetTable );
            }
            targetTable.put( tgtCurrency, rate );
        }
    }
    
    /**
     * Carries out the conversion.
     * If the table does not contain a conversion from the source currency to the target one,
     * a {@link ConversionResult} is returned that has null netAmount.
     * This implementation works with any {@link ConversionRequest}, it won't throw {@link IllegalRequestSubtypeException}.
     * @param req the conversion request
     * @return the conversion result
     */
    public ConversionResult convert( final ConversionRequest req ) {
        final Currency srcCurrency = req.getSrcAmount().getCurrency();
        final Currency tgtCurrency = req.getTgtCurrency();
        final ConversionRate rate = findConversionRate( srcCurrency, tgtCurrency );
        if ( rate != null ) {
            final MonetaryAmount tgtAmount = rate.convert( req.getSrcAmount() );
            return new ConversionResult( tgtAmount );
        } else {
            return new ConversionResult( null );    // did not find the pair of currencies in the table
        }
    }

    /**
     * Looks up the conversion between the given currencies in the table.
     * @param srcCurrency the source currency
     * @param tgtCurrency the target currency
     * @return the conversion rate; null means no conversion between the currencies was found in the table
     */
    protected ConversionRate findConversionRate( final Currency srcCurrency, final Currency tgtCurrency ) {
        synchronized ( conversionTable ) {
            final Map<Currency, ConversionRate> targetTable = conversionTable.get(srcCurrency);
            final ConversionRate rate = (targetTable != null) ? targetTable.get(tgtCurrency) : null;
            return rate;
        }
    }

}
